
/**
 * 
 * @param {Object} arg スレッド保存先ディレクトリ名かスレッドタイトル、もしくはFileオブジェクト
 */
function CacheFile(arg) {
	//定数
	this.MARGIN_OF_ERROR = 60 * 60 * 24 * 3; //誤差 三日前まで
	this.ALLOW_ERROR_COUNT = 100;
	
	//メンバ
	this.file = null;
	this.title = "";
	this.error = false;
	
	//----------------------
	
	try {
		if(arg && typeof(arg) == "string" && arg != "") {
			//var cachedir = getPref("moonfox.cachedir");
			//var join = cachedir.match(/\/$/) ? "" : "/";
			var cacheRootFile = getCacheRootFile();
			
			if(cacheRootFile && cacheRootFile.exist()) {
				if( arg.match(/^thread_/) ) { //ディレクトリ名から
					//var path = cachedir + join + arg;
					this.file = new File(cacheRootFile, arg);
					this.title = decodeTitle(arg);
				} else { //スレッドタイトルから
					var dirName = encodeTitle(arg);
					//var path = cachedir + join + dirName;
					this.file = new File(cacheRootFile, dirName);
				}
			} else {
				this.error = true;
			}
		} else if(arg && typeof(arg) == "object" && arg.toString().indexOf(CM_IO_FILE_ID) >= 0) {
			//Ｆｉｌｅオブジェクトから
			var nsIFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			nsIFile.initWithPath(arg.getPath());
			
			this.file = new File(nsIFile);
			this.title = decodeTitle(this.file.getName());
		}
	} catch(e) {
		throw e;
	}
	
	//----------------------
	/**
	 * 未読レコードがあるかチェックします。
	 */
	this.checkUnreadRecord = function() {
		var unread = false;
		
		var recordDir = this.getRecordDir();
		
		if(recordDir && recordDir.hasFile()) {
			var postHist = new PostHistory(this.file.getName());
			var targetStamp = parseInt(postHist.getLastStamp()) - this.MARGIN_OF_ERROR;
			
			if(targetStamp <= 0) {
				unread = true;
			} else {
				var recFileList = this.getRecordFileListByStamp(targetStamp);
				
				if(recFileList && recFileList.length > 0)
					unread = postHist.checkUnreadRecord(recFileList);
			}
		}
		
		return unread;
	}
	
	/**
	 * 
	 */
	this.isError = function() {
		return this.error;
	}
	
	/**
	 * このキャッシュが存在している場合trueを返します。
	 */
	this.exist = function() {
		var exi = this.file && this.file.exist();
		return exi;
	}
	
	/**
	 * 添付ファイルを格納するディレクトリのFileオブジェクトを返します。
	 *
	 * @return
	 */
	this.getAttachDir = function() {
		var attachDir = null;
		
		if(this.file && this.file.exist())
			attachDir = new File(this.file, "attach");
		
		return attachDir;
	}
	
	/*
	 * 添付ファイルの配列を返します。
	 *
	 * @param ext_array 拡張子リスト
	 * @return
	 */
	this.getAttachFileList = function(ext_array) {
		var fileList = new Array();
		var attachDir = this.getAttachDir();
		
		if(!attachDir || !attachDir.exist())
			return fileList;
		
		var candidates = attachDir.listFile();
		
		for(var i = 0; i < candidates.length; i++) {
			var ext = candidates[i].getExt();
			
			if(!ext_array) {
				fileList.push(candidates[i]);
			} else {
				if(contain(ext_array, ext)) {
					fileList.push(candidates[i]);
				}
			}
		}
		
		return fileList;
	}
	
	/**
	 * ディレクトリ名を返します。
	 *
	 * @return
	 */
	this.getDirName = function() {
		return this.file.getName();
	}
	
	/**
	 * ログに存在しないファイルを取得
	 */
	this.getNewRecordList = function(doc, lastRecord) {
		
		var recordList = new Array();
		var checkLimit = getPref("moonfox.check_limit_day") * 60 * 60 * 24;
		
		var recDir = this.getRecordDir();
		
		if(!recDir.exist())
			return recordList;
		
		var postHist = new PostHistory(this.file.getName());
		var targetStamp = null;
		var nextNo = 1;
		
		if(doc && lastRecord) {
			targetStamp = parseInt(lastRecord.stamp) - checkLimit;
			nextNo = parseInt(lastRecord.no) + 1;
		}
			
		var recFileList = this.getRecordFileListByStamp(targetStamp);
		
		for(var i = 0; i < recFileList.length; i++) {
			var name = recFileList[i].getName();
			
			if(!doc 
				|| (!postHist.contain(name) && !doc.getElementById(name)) ) {
				var record = parseRecord(recFileList[i], this.file.getName());
				record.no = nextNo;
				record.unread = true;
				recordList.push(record);
				nextNo += 1;
			}
		}
		
		return recordList;
	}
	
	/**
	 * 
	 */
	this.getRecordByBlockId = function(blockId) {
		var record = null;
		var recordFile = null;
		var recordFileList = this.getRecordFileListByStamp(null);
		
		for(var i = 0; i < recordFileList.length; i++) {
			var name = recordFileList[i].getName();
			
			if(name.indexOf(blockId) >= 0) {
				recordFile = recordFileList[i];
				break;
			}
		}
		
		if(recordFile) {
			record = parseRecord(recordFile);
			record.no = -1;
		}
		
		return record;
	}
	
	/**
	 * 朔のレコード保存ディレクトリを返します。
	 *
	 * @return
	 */
	this.getRecordDir = function() {
		var recordDir = null;
		
		if(this.file && this.file.exist())
			recordDir = new File(this.file, "record");
		
		return recordDir;
	}
	
	/**
	 * stamp以降のレコードファイルの配列を返します。
	 * stampがnullのときは全ファイル。
	 * 
	 * @param {Object} stamp
	 */
	this.getRecordFileListByStamp = function(stamp) {
		var recordFile = this.getRecordDir();
		var recListByStamp = new Array();
		
		if(!recordFile || !recordFile.exist())
			return recListByStamp;
		
		var fileList = recordFile.listFile();
		
		for(var i = fileList.length - 1; i >= 0; i--) {
			var name = fileList[i].getName();
			
			if( stamp == null 
					|| (name.match(/^([0-9]+)\_.+/) && parseInt(RegExp.$1) >= stamp) ) {
				recListByStamp.unshift(fileList[i]);
				
			} else {
				break;
			}
		}
		
		return recListByStamp;
	}
	
	/**
	 * 
	 */
	this.getThreadName = function() {
		var name = "";
		
		if(this.file) {
			name = this.file.getName();
		}
		
		return name;
	}
	
	/**
	 * スレッドのタイトルを返します。
	 *
	 * @return
	 */
	this.getTitle = function() {
		return this.title;
	}
	
	/**
	 * 
	 */
	this.toString = function() {
		return "CacheFile@moonfox";
	}
}