
/**
 * Moonfoxのログ操作クラス
 * 
 * 過去ログファイル名 ： 先頭記事stamp_(full|vacant).xml
 * 追加が可のときは
 * 
 * @param {Object} arg
 */
function MfLog(arg) {
	//定数
	this.RECENT_LOG_NAME = "0_recent.xml";
	this.RECENT_LOG_SIZE = 200;
	this.MF_LOG_SIZE = 1000;
	this.VACANT = "vacant";
	this.FULL = "full";
	

	//メンバ
	this.file = null;
	this.threadName = null;
	this.title = "";
	
	//----------------------
	
	try {
		if(arg && typeof(arg) == "string" && arg != "") {
			var logroot = getMoonfoxLogRoot();
			
			if(!logroot.exist())
				logroot.mkdir();
			
			if( arg.match(/^thread_/) ) { //ディレクトリ名から
				this.file = new File(logroot, arg);
				this.title = decodeTitle(arg);
				this.threadName = arg;
			} else { //スレッドタイトルから
				this.title = arg;
				var dirName = encodeTitle(arg);
				this.file = new File(logroot, dirName);
				this.threadName = dirName;
			}
			
			if(!this.file.exist())
				this.file.mkdir();
			
		}
	} catch(e) {
		throw e;
	}
	//--------------------------------------------------
	/**
	 * 
	 */
	this.getLogHistoryFile = function() {
		var logFile = new File(this.file, "log_history.xml");
		return logFile;
	}
	
	/**
	 * 
	 */
	this.getLogHistoryDoc = function() {
		var doc = null;
		var logFile = this.getLogHistoryFile();
		
		if(logFile.exist())
			doc = parseXmlFile(logFile);
			
		return doc;
	}
	
	/**
	 * 存在するログファイルの名称を配列として返します。
	 * @return
	 */
	this.getNameOfLogFiles = function() {
		var nameList = new Array();
		
		if(this.file && this.file.exist()) {
			var fileList = this.file.listFile();
			
			for(var i = 0; i < fileList.length; i++) {
				var name = fileList[i].getName();
				
				if( name.match(/^[0-9]+\_full|vacant\.xml$/) ) {
					nameList.unshift(name);
				}
			}
		}
		
		return nameList;
	}
	
	/**
	 * 
	 * @param {Object} stamp
	 */
	this.getNextLogList = function(stamp) {
		var recordList = null;
		
		var nextFile = this.getNextLogFile(stamp);
		
		if(nextFile && nextFile.exist()) {
			recordList = loadMoonfoxLog(nextFile);
		}
		
		return recordList;
	}
	
	/**
	 * 先頭記事の日付がstamp以降のログファイルを返します。
	 * @param {Object} stamp
	 */
	this.getNextLogFile = function(stamp) {
		var nextFile = null;
		var logFileList = this.file.listFile();
		
		for(var i = 0; i < logFileList.length; i++) {
			var name = logFileList[i].getName();
			
			if( name.match(/^([0-9]+)\_[0-9]+\.xml/) ) {
				var start = parseInt(RegExp.$1);
				
				if(start >= stamp) {
					nextFile = logFileList[i];
					break;
				}
			}
		}
		
		return nextFile;
	}
	
	/**
	 * 最新ログをDocumentオブジェクトとして返します。
	 */
	this.getRecentLogAsDoc = function() {
		var doc = null;
		
		var recentFile = this.getRecentLogFile();
		
		if(recentFile && recentFile.exist()) {
			doc = parseXmlFile(recentFile);
		}
		
		return doc;
	}
	
	/**
	 * 直近のログをRecordオブジェクトの配列で返します。
	 * 
	 */
	this.getRecentLogList = function() {
		var recordList = null;
		
		var doc = this.getRecentLogAsDoc();
		recordList = docToRecordList(doc);
		
		return recordList;
	}
	
	/**
	 * 
	 */
	this.getRecentLogFile = function() {
		var recentFile = new File(this.file, this.RECENT_LOG_NAME);
		
		return recentFile;
	}
	
	/**
	 * 過去ログをDocumentとして返します。
	 */
	this.getPastLogAsDoc = function(fileName) {
		var doc = null;
		
		if(this.file && this.file.exist) {
		
			var pastLogFile = new File(this.file, fileName);
		
			if(pastLogFile && pastLogFile.exist()) {
				doc = parseXmlFile(pastLogFile);
			}
		}
		
		return doc;
	}
	
	/**
	 * 
	 * @param {Object} stamp
	 */
	this.getPrevLogList = function(stamp) {
		var recordList = null;
		
		var prevFile = this.getPrevLogFile(stamp);
		
		if(prevFile && prevFile.exist()) {
			recordList = loadMoonfoxLog(prevFile);
		}
		
		return recordList;
	}
	
	/**
	 * 最終書き込みの日付がstamp以前のログファイルを返します。
	 * @param {Object} stamp
	 */
	this.getPrevLogFile = function(stamp) {
		var prevFile = null;
		var logFileList = this.file.listFile();
		
		for(var i = logFileList.length - 1; i >= 0 ; i--) {
			var name = logFileList[i].getName();
			
			if( name.match(/^[0-9]+\_([0-9]+)\.xml/) ) {
				var end = parseInt(RegExp.$1);
				
				if(end <= stamp) {
					prevFile = logFileList[i];
					break;
				}
			}
		}
		
		return prevFile;
	}
	
	/**
	 * 
	 */
	this.getVacantLogFile = function() {
		var vacFile = null;
		
		var fileList = this.file.listFile();
		
		if(fileList && fileList.length > 0) {
			var candidate = fileList[fileList.length - 1];
			var name = candidate.getName();
			
			if(name.match(/^[0-9]+\_vacant.xml/))
				vacFile = candidate;
		}
		
		return vacFile;
	}
	
	/**
	 * 
	 * @param {Object} recordList
	 */
	this.saveRecentLog = function(recordList) {
		var removedRecordList = null;
		
		if(recordList.length > this.RECENT_LOG_SIZE) {
			removedRecordList = new Array();
			var removeSize = recordList.length - parseInt(this.RECENT_LOG_SIZE / 2);
			
			for(var i = 0; i < removeSize; i++) {
				removedRecordList.push(recordList.shift());
			}
		}
		
		//最新ログを保存
		var recentFile = this.getRecentLogFile();
		writeRecordList(this.threadName, recentFile, recordList);
		
		//過去ログに保管
		if(removedRecordList) {
			this.savePastLog(removedRecordList);
		}
	}
	
	/**
	 * 
	 * @param {Object} recordList
	 */
	this.savePastLog = function(recordList) {
		
		while(recordList.length > 0) {
			var vacFile = this.getVacantLogFile();
			
			var saveRecordList = new Array();
			var capa = this.MF_LOG_SIZE;
			
			if(!vacFile) {
				vacFile = new File(this.file, recordList[0].stamp + "_" + this.VACANT + ".xml");
			} else {
				var doc = parseXmlFile(vacFile);
				saveRecordList = docToRecordList(doc);
				capa = this.MF_LOG_SIZE - saveRecordList.length;
			}
			
			for(var i = 0; i < capa; i++) {
				if(!recordList[0])
					break;
				
				saveRecordList.push(recordList.shift());
			}
			
			//書き出す
			writeRecordList(this.threadName, vacFile, saveRecordList);
			
			//規定記事数に達したら名称変更
			if(saveRecordList.length >= this.MF_LOG_SIZE) {
				var newName = vacFile.getName().replace(this.VACANT, this.FULL);
				vacFile.renameTo(newName);
			}
		}
	}
	
	/**
	 * 既に存在するログを更新します。
	 * @param {Object} recordList
	 * @param {Object} fileName
	 */
	this.updateLog = function(recordList, fileName) {
		if(fileName == "recent")
			fileName = this.RECENT_LOG_NAME;
		
		if(!this.file.exist())
			throw "ログが存在しません。\n" + this.file.getPath();
			
		var updateFile = new File(this.file, fileName);
		
		if(!updateFile.exist())
			throw "ファイルが存在しません。" + updateFile.getPath();
		
		writeRecordList(this.threadName, updateFile, recordList);
	}
}